use nom::branch::alt;
use nom::bytes::complete::tag;
use nom::character::complete::anychar;
use nom::character::complete::line_ending;
use nom::character::complete::space0;
use nom::combinator::eof;
use nom::combinator::opt;
use nom::combinator::recognize;
use nom::multi::many_till;
use nom::sequence::tuple;

use super::org_source::OrgSource;
use super::sexp::sexp;
use crate::context::RefContext;
use crate::error::Res;
use crate::parser::util::get_consumed;
use crate::parser::util::start_of_line;
use crate::types::DiarySexp;

#[cfg_attr(feature = "tracing", tracing::instrument(ret, level = "debug"))]
pub fn diary_sexp<'b, 'g, 'r, 's>(
    _context: RefContext<'b, 'g, 'r, 's>,
    input: OrgSource<'s>,
) -> Res<OrgSource<'s>, DiarySexp<'s>> {
    start_of_line(input)?;
    let (remaining, _leading_whitespace) = space0(input)?;
    let (remaining, _clock) = tag("%%")(remaining)?;
    let (remaining, _gap_whitespace) = space0(remaining)?;
    let (remaining, _sexp) = recognize(sexp)(remaining)?;
    let (remaining, _trailing_comment) = opt(tuple((
        space0,
        tag(";"),
        many_till(anychar, alt((line_ending, eof))),
    )))(remaining)?;
    let (remaining, _trailing_whitespace) =
        recognize(tuple((space0, alt((line_ending, eof)))))(remaining)?;

    let source = get_consumed(input, remaining);
    Ok((
        remaining,
        DiarySexp {
            source: source.into(),
        },
    ))
}
